#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/*	Rev03 Sequential Sinusoidal Wave Processing Function is added */
/*	Rev02 unuseful output files are taken off */
/*	Rev01 # of cycles option is dropped for consistency */
/*	Rev00 - combined program of simulator rev 10 and DFT rev 14 */


	const double pi=3.141592;
	const double p0=101300.0;			/* absolute atmospheric pressure */
	const double rho=1.2;					/* density of air */
	double *wave;									/* pointer array of simulated vector */

	int choice;										/* input signal selector choice option :
							 										  0: sinsoidal, 1: sweep, other: random input */
	int vvso;											/* velocity vector sum option, 0:all, 1:surface only */

	int i,j,k,n,ND,n_cycles,division,n_rows;
																/* counter, counter, counter, number of chamber,
																   number of calculation cycles,
							  									 number of divisons in a term, n_cycles*division */
	int req_cycles;								/* required cycles for DFT analysis */

	int hf, ndps, np1, progress, t_window, freq;
																/********** - DFT variables -
																	hf:		highest frequency to compute
																	ndps:		number of datapoints to read from file
																	np1:		n+1
																	progress:	show progress
																	t_window:	time window; this is equivalent to 2pi
																	freq:		
																**********/
	int seqsw, fseq, fsmin, fsmax;	/* switch, sequentially processed frequency, 
																		lowest frequency, highest frequency -Rev03- */

	double Q[25][25],KH[25][25],K[25][25];
																/* for recursive formula, standard stiffness matrix,
																   actual stiffness matrix */
	double L[25],M[25],A[25],R[25],V[25],force[25];
																/* duct length, duct mass, area, area ratio,
																   chamber volume, input force */
	double C[25];									/* damping coefficients : rev.07 */
	double MP[25], MM[25], MK[25][25];	/* modified mass with damping effect */
	double xp[25],xc[25],xn[25],sumx;		/* displacement vectors */
	double vc[25],sumv;						/* velocity vectors */
	double KS[13];								/* standard stiffness */
	double m0,radius,f0,F,phi,b,T,tempo,forte;
																/* membranee mass,radius of membrane,f0,
																   input frequency, sweep speed [Hz/s],
																   amplitude of input, time, force */
	double Ku, HCR, delta;				/* stiffness of sp unit, thermal condition,
																		time resolution */

	double tw, vel, delta_dft, frequenza, fres, dftr, dfti, dftp, dftl, phase;
																/**********	-DFT variables -
																	tw:					actual time window to avoid t_window=0
																	vel:				intermediate variable of velocity read from DFT input file
																	delta_dft:	modified time resolution to compute integral part ot DFT
																	frequenza:		
																	fres:				frequency resolution; computed value
																	dstr:				real part of transformed result
																	dfti:				imaginery part of transformed result
																	dftp:				sound pressure [Pa]
																	dftl:				sound pressure level[dB]
																	phase:			phase in degree
																**********/

	FILE *input;									/* simulation condition */
	FILE *displacement, *displacement_n;		/* raw displacement output & normalized displacement output */
	FILE *dft_v;									/* sum of normalized velocity without time column for DFT */
	FILE *velocity, *velocity_n;	/* velocity of each mass & normalized velocity of each mass */
	FILE *dft_params;							/* used condition values for humanbeing's view */
	FILE *matrix;									/* matrices K, RKR, M-1(RKR), Q */
	FILE *dft_input, *dft_output;	/* DFT input, DFT outpput */
	FILE *dft_seqoutput;					/* DFT output for sequential process */

/****** definition of normalizing: displacement & velocity of membrane are set reference value then
	ratio of area of each duct is multiplied to each displacement & velocity values 		******/

int results(void),matrices(void);		/* recursive computation, matrices print */
int dft(void), point_dft(void);			/* Discrete Fourier Trandform */

/*************************************************************************************************/
/*	DFT COMPUTATION FOR SPECIFIED FREQUENCY	*/
/*************************************************************************************************/
int point_dft()
{
	fscanf(dft_input,"%d\n",&ndps);			/* number of datapoints is read from file */
	fscanf(dft_input,"%lf\n",&delta);			/* delta is read from 2nd line of the file */

	tw=ndps*delta;
	if(tw>0.9)
	{
		if(tw<1.1)
		{
			tw=1.00001;
		}
	}

	t_window=int(tw);				/* compute time window T */
	fres=1/t_window;				/* frequency resolution */

/*	MEMORY ALLOCATION PROCESS */

	np1=ndps+1;					/* number of data points */
	wave = (double*)malloc (sizeof(double) * np1);	/* dynamic memory allocation */

/*	INITIAL VALUE SETTING FOR wave[i] */
	for(i=0;i<np1;i++){wave[i]=0;};

/*	READ VELOCITY VALUES FROM INPUT FILE */
	for(i=0;i<ndps;i++)
	{
		fscanf(dft_input,"%lf\n",&vel);
		wave[i]=vel;
	}

//	
	delta_dft=2*pi/ndps;	/* modified time resolution to compute integral for DFT */

	freq=fseq*t_window;
	frequenza=(double)(fseq)/(double)(t_window);

	dftr=0;
	dfti=0;

	 for(k=0;k<ndps;k++)
	{
		dftr=dftr + wave[k] *2*pi/ndps*cos(delta_dft*fseq*k);
		dfti=dfti - wave[k] *2*pi/ndps*sin(delta_dft*fseq*k);
		phase=atan(dfti/dftr)*180/pi;
	}

		dftp=0.5*rho*(pow(dftr,2)+pow(dfti,2));
		dftl=10*log(dftp/(2e-5));
		fprintf(dft_seqoutput,"%f,%f,%f,%f,%f,%f\n",frequenza,dftl,phase,dftp,dftr,dfti);
		printf("%d [Hz]\t%f\n",fseq,dftl);

	free(wave);

	return(1);

}
/*************************************************************************************************/
/*	DFT COMPUTATION FOR SPECIFIED FREQUENCY	RANGE */
/*************************************************************************************************/

int dft()
{
	dft_output=fopen("dft_result.csv","w");		/* output file discrete frequencies, real, imaginary */

	fscanf(dft_input,"%d\n",&ndps);			/* number of datapoints is read from file */
	printf("number of data points =%d\n",ndps);
	fscanf(dft_input,"%lf\n",&delta);			/* delta is read from 2nd line of the file */
	printf("time resolution =%f\n",delta);

	tw=ndps*delta;
	if(tw>0.9){
		if(tw<1.1){
			tw=1.00001;
		}
	}

	t_window=int(tw);				/* compute time window T */

	fres=1/t_window;				/* frequency resolution */

	printf("frequency resolution =\t%f\n\n",fres);

/*	printf("set upper frequency [Hz] (minimum is 40) : ");
	scanf("%d",&hf);*/

	hf=200;

	printf("time window = %d\n",t_window);
	
	if(hf>ndps){hf=ndps;}					/* any value if hf<40 then set hf=40 */
	if(hf<40){hf=40;}
	printf("lower frequency = 1\t[Hz]\n");
	printf("upper frequency = %d\t[Hz]\n",hf);

/*	MEMORY ALLOCATION PROCESS */

	np1=ndps+1;					/* number of data points */
	wave = (double*)malloc (sizeof(double) * np1);	/* dynamic memory allocation */

/*	INITIAL VALUE SETTING FOR wave[i] */
	for(i=0;i<np1;i++){wave[i]=0;};

/*	READ VELOCITY VALUES FROM INPUT FILE */
	for(i=0;i<ndps;i++){
		fscanf(dft_input,"%lf\n",&vel);
		  wave[i]=vel;
	}

//	
delta_dft=2*pi/ndps;			/* modified time resolution to compute integral for DFT */

/*	HEADER FOR OUTPUT FILE */
	fprintf(dft_output,"frequency[Hz],SPL,phase,Pa,Re,Im\n");	/* header */

	progress=-1;					/* progress counter */

	hf=hf*t_window;

	for(j=0;j<=hf;j++){
	
		freq=j*t_window;
		frequenza=(double)(j)/(double)(t_window);
	dftr=0;
	dfti=0;
		progress++;

	if (progress==100){

		printf("%d of %d steps done.\n",j,hf);
		progress=0;
	}
	  for(k=0;k<ndps;k++){
		dftr=dftr + wave[k] *2*pi/ndps*cos(delta_dft*j*k);
		dfti=dfti - wave[k] *2*pi/ndps*sin(delta_dft*j*k);
		phase=atan(dfti/dftr)*180/pi;
	  }
	dftp=0.5*rho*(pow(dftr,2)+pow(dfti,2));
	dftl=10*log(dftp/(2e-5));
	fprintf(dft_output,"%f,%f,%f,%f,%f,%f\n",frequenza,dftl,phase,dftp,dftr,dfti);
	}
	free(wave);

	fclose(dft_output);

	printf("DFT has been completed.\n");
	return(1);
}
/*************************************************************************************************/
/*	RECURSIVE CALCULATIONS */
/*************************************************************************************************/
int results(void)
{
//	THIS FUNCTION SEQUENTIALLY CALCULATE AND WRITE VECTORS

	dft_v=fopen("dftv.txt","w");							/* velocity output for DFT */

if(seqsw==1)	/* Skip these processes if sequential process option is taken */
{
//	WRITE HEADER ROW

//	HEADER FOR RAW VECTOR OUTPUT

		fprintf(displacement,"time[s],force[N]");
		fprintf(velocity,"time[s],force[N]");
		for (i=0;i<ND;i++){
			fprintf(displacement,",x[%d]",i);
			fprintf(velocity,",v[%d]",i);
		}
		if(choice==0){
			fprintf(displacement,",%f[Hz]\n",F);
			fprintf(velocity,",%f[Hz]\n",F);
		}
		else
		{
			fprintf(displacement,",random\n");
			fprintf(velocity,",random\n");
	}	

//	HEADER FOR NORMALIZED VECTOR OUTPUT

		fprintf(displacement_n,"time[s],sum_x[mm]");
		fprintf(velocity_n,"time[s],sum_v[m/s]");

		for (i=0;i<ND;i++){
			fprintf(displacement_n,",x_n[%d]",i);
			fprintf(velocity_n,",v_n[%d]",i);
		}
		if(choice==0){						/* sinusoidal wave */
			fprintf(displacement_n,",force[N],%f[Hz]\n",F);
			fprintf(velocity_n,",force[N],%f[Hz]\n",F);
		}
		else if(choice==1){					/* sweep signal */
			fprintf(displacement_n,",force[N],sweep rate[Hz/s]=%f,final frequency[Hz]=%f\n",phi,F);
			fprintf(velocity_n,",force[N],sweep rate[Hz/s],final frequency[Hz]\n",phi,F);
		}
		else{							/* random signal */
			fprintf(displacement_n,",force[N],random\n");
			fprintf(velocity_n,",force[N],random\n");
		}
}			/*	End "seqsw=1" option */

//	WRITE NUMBER OF POINTS AND DELTA T FOR DFT ANALYSIS

	fprintf(dft_v,"%d\n%lf\n",n_rows,delta);	/* 1st line: data points, 2nd line: delta t */

//	HEADER OUTPUT FOR UNUSED DFT FILES

//	END OF HEADER PROCESSES

	if(seqsw==1)
	{
		printf("Now recursive calculation process begins.\n");
	}
//	RECURSIVE STEP CALCULATION BEGINS

	for (k=0;k<=n_rows;k++){

	tempo=delta*k;						/* tempo is "time" */

//	SOURCE SIGNAL OPTION (0:SINUSOIDAL, 1:LINEAR SWEEP, 2:RANDOM)

	if(choice==0){
		forte=b*sin(2*pi*F*tempo);						/* sinusoidal signal input */
	}
	else if(choice==1){
		forte=b*sin(2*pi*phi*tempo*tempo);		/* sweep signal input */
	}
	else
	{
		forte=b*2*(rand()%100-50)/100;			/* random signal input */
	}

	force[0]=2*delta*delta/MP[0]*forte;			/* force is converted to recursive formula */
	

//	VECTOR SUM CALCULATION

	sumx=0;							/* initial value */
	sumv=0;

	for (j=0;j<ND;j++){sumx=sumx+xc[j]*R[j]*1E3;}		/* sum of normalized displacement */

//
		for (i=0;i<ND;i++){
			xn[i]=0.0;
			for (j=0;j<ND;j++){
			xn[i]=xn[i]+Q[i][j]*xc[j];
			}
			xn[i]=xn[i]-MM[i]/MP[i]*xp[i]+force[i];
			vc[i]=(xc[i]-xp[i])/delta;
		}

//	CHOOSE IF INTERNAL DUCTS ARE IGNORED

	if(vvso==0){
		vvso=0;						/* all ducts are used */
	}
	else{
		vvso=n;						/* internal ducts are ignored */
	}

	sumv=sumv+vc[0]*R[0];					/* velocity of membrane */
	for (j=vvso+1;j<ND;j++){sumv=sumv+vc[j]*R[j];}		/* velocity of ducts; internal ducts are ignored if vvso=1 */


//	WRITE RAW & NORMALIZED VECTORS

	fprintf(displacement,"%f,%f",tempo,forte);
	fprintf(displacement_n,"%f,%f",tempo,sumx);
	fprintf(velocity,"%f,%f",tempo,forte);
	fprintf(velocity_n,"%f,%f",tempo,sumv);

	for (i=0;i<ND;i++){

		fprintf(displacement,",%f",xc[i]*1E3);		/* raw vector */
		fprintf(displacement_n,",%f",xc[i]*R[i]*1E3);	/* normalized vector */
		fprintf(velocity,",%f",vc[i]);			/* raw vector (m/s)*/
		fprintf(velocity_n,",%f",vc[i]*R[i]);		/* normalized vector (m/s)*/

		xp[i]=xc[i];
		xc[i]=xn[i];

	}

	fprintf(displacement,"\n");
	fprintf(displacement_n,",%f\n",forte);			/* force vector is placed at last column */

	fprintf(velocity,"\n");
	fprintf(velocity_n,",%f\n",forte);			/* force vector is placed at last column */
	fprintf(dft_v,"%f\n",sumv);				/* output for dft analysis */
	}
	fclose(dft_v);
	return(1);
}

int matrices(void)
{
//	WRITE MATRIX VALUES FOR DEBUGGING PURPOSE ONLY

//	print "i" as header line

	for (i=0;i<ND;i++){
		fprintf(matrix,"%d,",i);
	}
	fprintf(matrix,"\n\n");

//	STANDARD STIFFNESS MATRIX

	fprintf(matrix,"standard stiffness matrix\n");
	for (i=0;i<ND;i++){
		for (j=0;j<ND;j++){
			fprintf(matrix,"%f,",KH[i][j]);
		}
	fprintf(matrix,"\n");
	}
	fprintf(matrix,"\n");

//	ACTUAL STIFFNESS MATRIX

	fprintf(matrix,"actual stiffness matrix\n");

	for (i=0;i<ND;i++){
		for (j=0;j<ND;j++){
			fprintf(matrix,"%f,",K[i][j]);
		}
	fprintf(matrix,"\n");
	}
	fprintf(matrix,"\n");

//	Q MATRIX - INTERMEDIATE MATRIX FOR RECURSIVE CALCULATION -

	fprintf(matrix,"Q matrix\n");
	for (i=0;i<ND;i++){
		for (j=0;j<ND;j++){
			fprintf(matrix,"%f,",Q[i][j]);
		}
	fprintf(matrix,"\n");
	}

	return(1);
}
/**************************************************************************************************/
/*  MAIN FUNCTION */
/**************************************************************************************************/
main()
{
	input=fopen("parameters.txt","r");				/* input parameter file */
	displacement=fopen("x_vectors.csv","w");		/* raw displacement vector */
	displacement_n=fopen("normalized_x_vectors.csv","w");	/* normalized displacement vector */
	
	velocity=fopen("v_vectors.csv","w");			/* raw velocity vector */
	velocity_n=fopen("normalized_v_vectors.csv","w");	/* normalized velocity vector */
	dft_params=fopen("dft_parameters.txt","w");		/* output for DFT analysis */
	
	matrix=fopen("matrices.csv","w");			/* stiffness and other matrices */

	dft_seqoutput=fopen("dft_sequential_calculation_result.csv","w");
		/* output file discrete frequencies, real, imaginary */

//	READ INPUT PARAMETERS FROM FILE

	fscanf(input,"%lf\t%lf\t%lf\n",&m0,&radius,&f0);	/* unit's spec */
	fscanf(input,"%d\t%d\t%lf\t%lf",&n,&division,&HCR,&b);

//	PRIMARY VALUES CALCULATION IN OTDER TO READ OTHER VALUES FROM INPUT CONDITION FILE

	ND=2*n+1;						/* number of degrees of freedom */

//	READ REMAINING INPUT PARAMETERS FROM FILE

	for (i=0;i<=n;i++){fscanf(input,"%lf\t",&V[i]);}	/* volume of chambers */
	for (i=1;i<ND;i++){fscanf(input,"%lf\t",&A[i]);}	/* area of ducts */
	for (i=1;i<ND;i++){fscanf(input,"%lf\t",&L[i]);}	/* length of ducts */
	for (i=0;i<ND;i++){fscanf(input,"%lf\t",&C[i]);}	/* damping coefficients */
								/* n, cycles, heat capacity ratio, force */

//	INTERMEDIATE PARAMETER VALUES

	A[0]=pi*pow(radius,2);					/* area of membrane */
	M[0]=m0*1.0E-3;						/* mass of membrane unit conversion to [kg] */
	L[0]=m0*1.0E-3/A[0]*10000.0/1.2*1000;			/* not used anywhere but put into memory */
	R[0]=1.0;						/* A[0]/A[0]=1 */
	Ku=4.0*pow(pi,2)*pow(f0,2)*M[0];			/* spring constant of sp unit */

printf("[0] values and parameters caluculation done!\n");

//	CHOOSE USING ALL DUCT OR IGNORING INTERNAL DUCTS FOR VELOCITY VECTOR SUM CALCULATION

	printf("Choose vector sum option\n");
	printf("\t 0: All ducts, 1: ignore internal ducts\n");
	printf("Your choice =\t");
	scanf("%d",&vvso);


//	CHOOSE INPUT SIGNAL OPTIONS

	printf("Choose Sequential Processing or One time Processing\n");					/* Rev03 function */
	printf("\tSequential Processing:\t0\n\tOne Time Processing:\t1\n");
	printf("You choose :");
	scanf("%d",&seqsw);
	printf("\n");

	if(seqsw==1)
	{
		printf("Choose Sinusoidal, Sweep or Random\n\n");
		printf("\t0:Sinusoidal, 1:Linear Sweep, 2:Random\n");
		printf("Your choice =\t");
		scanf("%d",&choice);
		printf("\n");
	}
	else{
			choice=0;		/* Same calculation as sinusoidal wave option */
			fsmin=10;
			fsmax=400;
			fseq=fsmin;
			n_cycles=fseq;
			n_rows=n_cycles*division;
			F=fseq;
			T=1/F;
			delta=T/division;

			fprintf(dft_seqoutput,"frequency[Hz],SPL,phase,Pa,Re,Im\n");	/*	HEADER FOR OUTPUT FILE */

			goto seqoption;
}
	if(choice==0){
		printf("input frequency[Hz]=");
		scanf("%lf",&F);			/* external force frequency */
		req_cycles=F;				/* compute required cycles */
		n_cycles=req_cycles;			/* rev 05 */
		n_rows=n_cycles*division;		/* number of data rows */
		T=1/F;					/* term of wave in [s]*/
		delta=T/division;			/* time resolution */

	printf("You have chosen sinsoidal wave at %f [Hz].\n",F);
	}
	else if(choice==1){				/* processing sweep signal */
		
		printf("Sweep rate is set as highest frequency per second.\n");
		printf("This is to simplify DFT analysis; time window is always 1[s].\n");
		printf("highest frequency[Hz]=");
		scanf("%lf",&F);

		if(F>300){F=300;}			/* limit maxmum frequency is 300[Hz] */
		if(F<200){F=200;}			/* at least sweep to 200[Hz] */

		phi=F;				/* Sweep rate = Higest Frequenct per second */

		T=1/F;					/* term of wave in [s]*/
		delta=T/division;			/* time resolution */

		n_rows= (int)(F/phi/delta);		/* update number of cycles to cover max frequency */

		printf("Number of Rows = %d\n",n_rows);
		printf("You have chosen linear sweep at %f [Hz/s] up to %f [Hz].\n",phi,F);
	}
	else{					/* wrong values are assumed random signal was chosen */

		F=100;					/* 100Hz is assumed for random signal analysis
							time resolution = 1/100/division for random */
		req_cycles=F;				/* assuming 100Hz signal */
		n_cycles=req_cycles;
		n_rows=n_cycles*division;		/* number of data rows */
		T=1/F;					/* term of wave in [s]*/
		delta=T/division;			/* time resolution */

		printf("You have chosen random signal.\n");
	}

seqoption:		/* Regular Process was Skipped */

//	CHAMBER VOLUME UNIT CONVERSION FROM LITRE TO CUBIC-METRE

	for (i=0;i<=n;i++){V[i]=V[i]/1000.0;}

printf ("chamber volume unit conversion done!\n");

	for (i=1;i<ND;i++){
	R[i]=A[i]/A[0];						/* ratio of areas without unit conversion */
	M[i]=A[i]*1.0E-4*L[i]*1.0E-3*1.2;			/* mass vector calculation [kg] */
	}
	printf("M[i]\tC[i]\n");														/* test purpose only */
	for(i=0;i<ND;i++){printf("%f\t%f\n",M[i],C[i]);}	/* test purpose only */

	for (i=0;i<ND;i++){
	MP[i]=2*M[i]+delta*C[i];			/* modified mass matrix for damping compensation */
	MM[i]=2*M[i]-delta*C[i];			/* modified mass matrix for damping compensation */
	}

// CHEACK ONLY
	printf("i\tR[i]\tA[i]\tL[i]\tM[i]\n");
	for (i=0;i<ND;i++){printf ("%d\t%f\t%f\t%f\t%f\n",i,R[i],A[i],L[i],M[i]);}
	for (i=0;i<ND;i++){printf ("%f\t%f\n",MM[i],MP[i]);}	/* test purpose only */
	printf("\n-- damping effects below --\n");
	for (i=0;i<ND;i++){printf ("%f\t",(1.0-MM[i]/MP[i]));}
	printf("\n");

printf("ratio and mass vector calculation done!\n");

	for (i=0;i<=n;i++){KS[i]= HCR*A[0]*A[0]*1E-8*p0/V[i];
	printf("standard stiffness matrix values %d\t%f\n",i,KS[i]);}
	printf("\n");

printf("read data stored to matrices!\n");

//	INITIAL VALUE SETTING
	for (i=0;i<25;i++){
		xp[i]=0.0;		/* previous displacement vector */
		xc[i]=0.0;		/* current displacement vector */
		xn[i]=0.0;		/* next displacement vector */
		force[i]=0.0;		/* force vector */
//
		for (j=0;j<25;j++){
		KH[i][j]=0.0;		/* standard stiffness matrix */
		K[i][j]=0.0;		/* actual stiffness matrix */
		Q[i][j]=0.0;		/* intermediate matrix for calculation */
		}
	}
//	START STANDARD STIFFNESS MATRIX CALCULATION

	for (i=0;i<=n;i++){
		for (j=0;j<=n;j++){
		KH[i][j]=KS[0];		/* upper left block */
		}
	}
printf("upper left block k0 done!\n");

	KH[0][0]=KH[0][0]+Ku;
	for (i=1;i<=n;i++){
		KH[i][i]=KH[i][i]+KS[i];	/* diagonal element of upper left block */
	}
	for (i=1;i<=n;i++){
		KH[n+i][i]=-KS[i];	/* lower left block */
		KH[i][n+i]=-KS[i];	/* upper right block */
	}
	for (i=1;i<=n;i++){
		KH[n+i][n+i]=KS[i];	/* lower right block */
	}
//	END STANDARD STIFFNESS MATRIX CALCULATION

printf("standard stiffnes matrix is ready!\n");

//	START ACTUAL STIFFNESS CALCULATION MATRIX

	for (i=0;i<ND;i++){
		for (j=0;j<ND;j++){
			K[i][j]=KH[i][j]*R[i]*R[j];
		}
	}
//	END ACTUAL STIFFNESS MATRIX CALCULATION

printf("actual stiffnes matrix is ready!\n");


//	MK[i][j]=2*(2*M[i]-delta^2*K[i][j]) rev07 and later
//	 -- this matrix is to calculate coefficient matrix of x[j] --

	for(i=0;i<ND;i++){for(j=0;j<ND;j++){MK[i][j]=0.0;}}	/* initial value = 0 */

	for(i=0;i<ND;i++){
		MK[i][i]=M[i];					/* diagonal elements are given */
	}
	for(i=0;i<ND;i++){
		for(j=0;j<ND;j++){
			MK[i][j]=2*(2*MK[i][j]-pow(delta,2)*K[i][j]); /* all elements */
		}
	}

//	Calculate "Q=MK/MP" rev07 and later
//	 -- this is coefficient matrix of x[j] --

	for (i=0;i<ND;i++){
		for (j=0;j<ND;j++){
		Q[i][j]=MK[i][j]/MP[i];
		}
	}

	printf("ready to begin simulation!\n");

	results();	/* calculate recursive formulae and save in a file */

if(seqsw==0)
{
	for(fseq=fsmin;fseq<=fsmax;fseq++)
		{
			F=fseq;
			results();
			dft_input=fopen("dftv.txt","r");			/* input file was created by previous program */
			point_dft();
			fclose(dft_input);
		}
		goto closure;
}
  fprintf(dft_params,"base frequency=%f\nnumber_of_cycles=%d\ndivision_per_cycle=%d\ndelta=%f",F,n_cycles,division,delta);
	/* parameter output for DFT */ 

  printf("%f\t%d\t%d\t%f\n",F,n_cycles,division,delta);

	printf("Dynamic simulation became complete!\n");

	matrices();	/* save matrix files for debugging */
	printf("Matrces have been recorded.\n");

	printf(" Now DFT calculation begins\n");

	dft_input=fopen("dftv.txt","r");			/* input file was created by previous program */

	dft();
	fclose(dft_input);

closure:

fclose(input);
fclose(displacement);
fclose(displacement_n);
fclose(velocity);
fclose(velocity_n);
fclose(matrix);
fclose(dft_params);
fclose(dft_seqoutput);
}